/*************************************************************************************
Mat 259 Project 2 3D Visualization: "Mental Health from Checkouts over time"               
Tested in Processing 4                                  
                                                     
Author: Jing Peng                       
Supervisor: George Legrady                           

Purpose: Show how to present volume data in a basic 3D environment     

Usage: 1. A mouse left-drag will rotate the camera around the subject.
          2. A right drag will zoom in and out. 
          3. A middle-drag (command-left-drag on mac) will pan. 
          4. A double-click restores the camera to its original position. 
          5. The shift key constrains rotation and panning to one axis or the other.
*************************************************************************************/

import peasy.*;
PeasyCam cam;

int startYear = 2006;
int endYear = 2023;

Table table;
int numRows, numColumns;

ArrayList<Category> categories = new ArrayList<Category>();

void setup(){
  size(1000, 800, P3D);
  cam = new PeasyCam(this, 200);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(800);
  
  //Added on Jan 30th by Rodger Luo
  float cameraZ = (height/2.0) / tan(PI*60.0/360.0);
  perspective(PI/3.0, width/height, 0.001, cameraZ*10.0); //set near clip to 0.001 to go inside data
  //Finished adding 
  
  table = loadTable("categories_out.csv", "header");
  numRows = table.getRowCount();
  println("Rows: " + numRows);

  for(int i=0; i<numRows; i++){

    int year = table.getInt(i,0);
    int month = table.getInt(i,1);
    String date = table.getString(i,8);

    String category= "Anxiety";
    float percentage = table.getFloat(i,9);
    categories.add(new Category(year, month, date, category, percentage));
        System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);


    category= "Stress";
    percentage = table.getFloat(i,10);
    categories.add(new Category(year, month, date, category, percentage));
    System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

    category= "Panic";
    percentage = table.getFloat(i,11);
    categories.add(new Category(year, month, date, category, percentage));
    System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

    category= "Depression";
    percentage = table.getFloat(i,12);
    categories.add(new Category(year, month, date, category, percentage));
    System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

    category= "Mental_Health";
    percentage = table.getFloat(i,13);
    categories.add(new Category(year, month, date, category, percentage));

    System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);


  }
}

void draw(){
  background(100);

  for(int i=0; i<categories.size(); i++){
    float yearAxis = map(categories.get(i).year, 2006, 2023, -50, 50);
    // float monthAxis = categories.get(i)+0.000;
    float monthAxis = map(categories.get(i).month, 1, 12, -50, 50);
    float percentageAxis = map(categories.get(i).percentage, 0, 1.1, -50, 50);
    color c = elementDetermine(categories.get(i).category);

    // We only draw the points that are later than 2005-12-31
    if (yearAxis > map(2006, 2006, 2023, -50, 50)){
        pushMatrix();
        translate(yearAxis, monthAxis, percentageAxis);
        strokeWeight(2);
        stroke(c);
        point(0,0,0);
        // compute the mouse distance with the point
        float mouseObjectDistance = sq(mouseX-screenX(0,0,0)) + sq(mouseY-screenY(0,0,0));
        // if the distance is smaller than a certain value
        if(mouseObjectDistance <20){
        // add a bigger point
        strokeWeight(10);
        point(0,0,0);
        fill(255,255,255);
        // add text label
        textMode(SHAPE);
        // if the distance is close, make the textsize smaller
        // if(cam.getDistance()<100){
        // textSize(3);}
        // // if the distance is far, make the textsize bigger
        // else{
        //     textSize(5);}
        String ca = " "+categories.get(i).percentage+"%"+categories.get(i).date;
        text(categories.get(i).category+ca,0,0);
        }
        popMatrix();
    }
  }
  
  drawLabels();  
    
  fill(200, 20);
  noStroke();
  box(100);
}

void drawLabels(){
  fill(255);
  textSize(30);
  pushMatrix();
  textAlign(LEFT, TOP);
  translate(-50, 50, 50);
  scale(0.1);
  text("2006", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(50, 50, 50);
  scale(0.1);
  text("2023", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(-50, -50, 50);
  scale(0.1);
  text("Dec", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, BOTTOM);
  translate(-50, 50, 50);
  scale(0.1);
  text("Jan", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(LEFT, CENTER);
  translate(50, 50, 50);
  scale(0.1);
  text("Percentage", 0, 0, 0);
  popMatrix();
  
//   pushMatrix();
//   textAlign(LEFT, CENTER);
//   translate(50, 50, -50);
//   scale(0.1);
//   text("Perta", 0, 0, 0);
//   popMatrix();
}
