/*************************************************************************************
Mat 259 Project 2 3D Visualization: "Mental Health from Checkouts over time"               
Tested in Processing 4                                  
                                                     
Author: Jing Peng                       
Supervisor: George Legrady                           

Purpose: Show how to present volume data in a basic 3D environment     

Usage: 1. A mouse left-drag will rotate the camera around the subject.
          2. A right drag will zoom in and out. 
          3. A middle-drag (command-left-drag on mac) will pan. 
          4. A double-click restores the camera to its original position. 
          5. The shift key constrains rotation and panning to one axis or the other.
*************************************************************************************/

import peasy.*;
import controlP5.*;

PeasyCam cam;
ControlP5 cp5;

int startYear = 2006;
int endYear = 2023;

float radiusMultiplier = 100; // Multiplier for percentage-based radius
float minRadius = 5; // Minimum radius for points
float maxRadius = 30; // Maximum radius for points
float radius;
PVector[] points;
color[] categoryColors;
int[] kind;


Table table;
int numRows, numColumns;

ArrayList<Category> categories = new ArrayList<Category>();
// Map<Integer, ArrayList<Category>> map = new HashMap<Integer,ArrayList<Category>>();

void setup() {
  size(1000, 800, P3D);
  cam = new PeasyCam(this, 200);
  // cam.setMinimumDistance(50);
  // cam.setMaximumDistance(800);
  cp5 = new ControlP5(this);
  
  //Added on Jan 30th by Rodger Luo
  // float cameraZ = (height/2.0) / tan(PI*60.0/360.0);
  // perspective(PI/3.0, width/height, 0.001, cameraZ*10.0); //set near clip to 0.001 to go inside data
  //Finished adding 
  categoryColors = new color[5];


  loadData();

  categoryColors[0] = color(255, 0, 0); // Red
  categoryColors[1] = color(0, 255, 0); // Green
  categoryColors[2] = color(0, 0, 255); // Blue
  categoryColors[3] = color(255, 255, 0); // Yellow
  categoryColors[4] = color(255, 0, 255); // Yellow

}

 void draw() {

   background(100);
   gui();

   noFill();
   stroke(0);
   sphere(radius);
   
  //   //Draw the data points
  //for (int i = 0; i < categories.size(); i++) {
  // PVector p = points[i];
  // int category = kind[i];
  // fill(categoryColors[category]);
  // float percentage = categories.get(i).percentage;
  // noStroke();
  // float scaledRadius = map(percentage, 0, 1, 5, 20);
  // pushMatrix();
  // translate(p.x, p.y, p.z);
  // sphere(scaledRadius);
  // popMatrix();
  //}

     // Draw the polytopes
   //for (int category = 0; category < 4; category++) {
   // stroke(categoryColors[category]);
   // beginShape();
   // for (int i = 0; i < numRows; i++) {
   //   if (kind[i] == category) {
   //     PVector p = points[i];
   //     vertex(p.x, p.y, p.z);
   //   }
   // }
   // endShape(CLOSE);
   //}
   
  
  for (int category = 0; category < 5; category++) {
    stroke(categoryColors[category]);
   beginShape();
   for (int i = 0; i < categories.size(); i++) {
      if (kind[i] == category) {
        PVector p = points[i];
        vertex(p.x, p.y, p.z);
      }
   }
      endShape(CLOSE);
  }

   if((mouseX<180) & (mouseY<180)) {
     cam.setActive(false);
   } else {
     cam.setActive(true);
   }

 }
 
 void gui() {
   hint(DISABLE_DEPTH_TEST);
   cam.beginHUD();
   cp5.draw();
   cam.endHUD();
   hint(ENABLE_DEPTH_TEST);
 }

 void loadData() {
   table = loadTable("categories_out.csv", "header");
   numRows = table.getRowCount();
   println("Rows: " + numRows);
   kind = new int[numRows * 5];
   
   for(int i=0; i<numRows; i++){
     int year = table.getInt(i,0);
     int month = table.getInt(i,1);
     String date = table.getString(i,8);

     String category= "Anxiety";
     float percentage = table.getFloat(i,9);
     categories.add(new Category(year, month, date, category, percentage));
     kind[i * 5] = 0;

         // System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

     category= "Stress";
     percentage = table.getFloat(i,10);
     categories.add(new Category(year, month, date, category, percentage));
          kind[i * 5+1] = 1;

     // System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

     category= "Panic";
     percentage = table.getFloat(i,11);
     categories.add(new Category(year, month, date, category, percentage));
          kind[i * 5+2] = 2;

     // System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

     category= "Depression";
     percentage = table.getFloat(i,12);
     categories.add(new Category(year, month, date, category, percentage));
    kind[i * 5+3] = 3;

     // System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);

     category= "Mental_Health";
     percentage = table.getFloat(i,13);
     categories.add(new Category(year, month, date, category, percentage));
      kind[i * 5+4] = 4;

     // System.out.println(year+" "+month+" "+date+" "+category+" "+percentage);
   }
   
   points = new PVector[categories.size()];
   
   //System.out.print(categories.size()+ " "+numRows);

   for (int i = 0; i < categories.size(); i++) {
     int year = int(categories.get(i).year);
     int month = int(categories.get(i).month);
     float percentage = categories.get(i).percentage;
     int category = int(categories.get(i).category);

     float theta = map(year, 2006, 2024, -PI, PI);
     float phi = map(month, 1, 12, -HALF_PI, HALF_PI);
     float r = map(percentage, 0, 0.2, minRadius, maxRadius) * radiusMultiplier;
     float x = r * sin(phi) * cos(theta);
     float y = r * sin(phi) * sin(theta);
     float z = r * cos(phi);

     points[i] = new PVector(x, y, z);
     // percentages[i] = percentage;
   }

   radius = minRadius * radiusMultiplier;
 }
